/*EE231002 Lab 16 Image Processing
  103061232.Chong-yao,Zhang Jian
  Date:2015.1.1
*/
#include <stdio.h>
#include <stdlib.h>
#define width 0
#define height 1

typedef struct sPIXEL {
	unsigned char r, g, b;	//three color components
}
PIXEL;

typedef struct sIMG {
	char header[3];	//header,either P3 or P6
	int W, H;	//width and height
	int level;	//intensity level of each color
	PIXEL **PX;	//2D array for all the pixels
} IMG;

IMG* PPMin(char *inFile);	//read image
void *PPMfout(IMG *p1, char *outFile);	//print image
IMG* PPMcvt(IMG *p1, IMG *ee, IMG *nthu, int x1, int y1, int x2, int y2);	//compose image

int main(int argc, char *argv[])
{
	IMG *back, *ee, *nthu;	//declare pointer points to image

	back = PPMin(argv[1]);	//read background photo
	ee = PPMin(argv[2]);	//read EE logo
	nthu = PPMin(argv[3]);	//read NTHU logo
	PPMcvt(back, ee, nthu, 910, 1780, 1090, 1600);	//compose image
	PPMfout(back, argv[4]);	//print image

	return 0;	//program end
}

IMG* PPMin(char *inFile)
{
	int i, j;	//looping index
	FILE *file;
	IMG *image = (IMG*) malloc(sizeof(IMG));

	file = fopen(inFile, "r");	//open file
	fscanf(file, "%s", image->header);	//scan P6
	fscanf(file, "%d %d\n%d\n", &image->W, &image->H, &image->level);	//scan info
	image->PX = (PIXEL **)malloc(image->W * sizeof(PIXEL *));
	for (i = 0; i < image->W; i++) {
		image->PX[i] = (PIXEL*)malloc(image->H * sizeof(PIXEL));
	}
	for (i = 0; i < image->H; i++) {
		for (j = 0; j < image->W; j++) {
			image->PX[j][i].r = fgetc(file);	//get pixel
			image->PX[j][i].g = fgetc(file);	//...
			image->PX[j][i].b = fgetc(file);	//...
		}
	}
	fclose(file);	//close file

	return image;	//return pointer
}

void *PPMfout(IMG *p1, char *outFile)
{
	int i, j;
	FILE *fout;

	fout = fopen(outFile, "w");	//open foutput file
	fprintf(fout, "%s\n", p1->header);	//write header
	fprintf(fout, "%d %d\n", p1->W, p1->H);	//write width &height
	fprintf(fout, "%d\n", p1->level);	//write color level
	for (i = 0; i < p1->H; i++) {
		for (j = 0; j < p1->W; j++) {
			fprintf(fout, "%c", p1->PX[j][i].r);
			fprintf(fout, "%c", p1->PX[j][i].g);
			fprintf(fout, "%c", p1->PX[j][i].b);
		}
	}
	fclose(fout);	//close foutput file

	return;
}

IMG* PPMcvt(IMG *p1, IMG *ee, IMG *nthu, int x1, int y1, int x2, int y2)
{
	int i, j,
		gray,
		eeX, eeY,
		nthuX, nthuY;

	for (j = 0; j < p1->H; j++) {
		for (i = 0; i < p1->W; i++) {
			if (i < x1 ||
				i > x2 ||
				j < y2 ||
				j > y1) {
				gray = p1->PX[i][j].r * 0.2126 +
					   p1->PX[i][j].g * 0.7152 +
					   p1->PX[i][j].b * 0.0722;
				p1->PX[i][j].r = gray;
				p1->PX[i][j].g = gray;
				p1->PX[i][j].b = gray;
			}
		}
	}

	eeX = (p1->W) - (ee->W);	//find location for EE logo
	eeY = (p1->H) - (ee->H);	//...
	for (j = 0; j < ee->H; j++) {
		for (i = 0; i < ee->W; i++) {
			if (ee->PX[i][j].r != 255 ||
				ee->PX[i][j].g != 255 ||
				ee->PX[i][j].b != 255) {
				p1->PX[eeX + i][eeY + j] = ee->PX[i][j];
			}
		}
	}

	nthuX = (p1->W - nthu->W) / 2;	//find location for NTHU logo
	nthuY = (p1->H - nthu->H) / 2;	//...
	for (j = 0; j < nthu->H; j++) {
		for (i = 0; i < nthu->W; i++) {
			if (nthu->PX[i][j].r != 255 ||
				nthu->PX[i][j].g != 255 ||
				nthu->PX[i][j].b != 255) {
				p1->PX[nthuX + i][nthuY + j].r = 255;
				p1->PX[nthuX + i][nthuY + j].b = 255;
			}
		}
	}

	for (j = (y2 - 1); j <= (y1 + 1); j++) {
		for (i = (x1 - 1); i <= (x2 + 1); i++) {
			if (i <= (x1 + 1) ||
				i >= (x2 - 1) ||
				j <= (y2 + 1) ||
				j >= (y1 - 1)) {
				p1->PX[i][j].r = 0;
				p1->PX[i][j].g = 255;
				p1->PX[i][j].b = 255;
			}
		}
	}

	return p1;
}
